<?php
/*
Plugin Name: GoodLocal Express
Plugin URI: https://goodlocalexpress.ca/
Description: Fast, easy and convenient delivery designed for local businesses. GoodLocal Express will pick up orders from your business and deliver them to your customers. Same day, next day, local and rural delivery services available!
Version: 1.0.0
Author: GoodLocal

Copyright (C) 2022  GoodLocal

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

For support, please contact info@goodlocal.ca!
*/

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

// Test to see if WooCommerce is active (including network activated).
$plugin_path = trailingslashit( WP_PLUGIN_DIR ) . 'woocommerce/woocommerce.php';

if (
    in_array( $plugin_path, wp_get_active_and_valid_plugins() )
    || in_array( $plugin_path, wp_get_active_network_plugins() )
) {
    function goodlocal_express_shipping_method_init() {
		if ( ! class_exists( 'WC_GL_Express_Local_Shipping_Method' ) ) {
			class WC_GL_Express_Local_Shipping_Method extends WC_Shipping_Method {
				/**
				 * Constructor for your shipping class
				 *
				 * @access public
				 * @return void
				 */
				public function __construct( $instance_id = 0 ) {
					$this->services = array(
						'GLE.LD'      	=> __( 'Local Delivery' ),
						'GLE.RD'        => __( 'Rural Delivery' ),
					);
					$this->id                 = 'goodlocal_express_local'; // Id for your shipping method. Should be unique.
					$this->instance_id 		  	= absint( $instance_id );
					$this->method_title       = __( 'GoodLocal Express' );  // Title shown in admin
					$this->method_description = __( '<p>The GoodLocal Express plugin gets rates dynamically from the GoodLocal Express API during cart/checkout.</p>
																					 <p>Use the <a href="https://goodlocalexpress.ca" target="_blank" style="text-decoration:none;">GoodLocal Express</a> app to request delivery for your orders.</p>
																					 <p>Need more help? Contact <a href="mailto:vendors@goodlocal.ca" style="text-decoration:none;">vendors@goodlocal.ca</a>.</p>' ); // Description shown in admin
					$this->supports           = array(
																				'shipping-zones',
																				'settings',
																				'instance-settings',
																				// 'instance-settings-modal',
																			);

					$this->init();
				}

				/**
				 * Initialize settings
				 *
				 * @return void
				 */
				private function set_settings() {
					$this->title               = $this->get_option( 'title', $this->method_title );
					$this->custom_services     = $this->get_option( 'services', array() );
				}

				/**
				 * Init your settings
				 *
				 * @access public
				 * @return void
				 */
				private function init() {
					// Load the settings.
					$this->init_form_fields();
					$this->set_settings();

					add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
				}

				/**
				 * Process settings on save.
				 *
				 * @access public
				 * @return void
				 */
				public function process_admin_options() {
					parent::process_admin_options();

					$this->set_settings();
				}

				/**
				 * generate_services_html function.
				 *
				 * @access public
				 * @return void
				 */
				public function generate_services_html() {
					ob_start();
					?>
					<tr valign="top" id="service_options">
						<th scope="row" class="titledesc"><?php _e( 'Services' ); ?></th>
						<td class="forminp">
						<style type="text/css">
								.goodlocal_express_services {
									width: 600px;
								}
								.goodlocal_express_services td {
									vertical-align: middle;
									padding: 4px 7px;
								}
								.goodlocal_express_services th {
									padding: 9px 7px;
								}
							</style>
							<table class="goodlocal_express_services widefat">
								<col style="width:50%">
								<col style="width:20%">
								<col style="width:30%">
								<thead>
									<th>Service Name</th>
									<th>Enabled</th>
									<th>Additional Charge</th>
								</thead>
								<tbody>
									<?php
										$add = 0;
										$this->ordered_services = array();

										foreach ( $this->services as $code => $name ) {
											$this->ordered_services[ $add ] = array( $code, $name );
											$add++;
										}

										foreach ( $this->ordered_services as $value ) {
											$code = $value[0];
											$name = $value[1];
											?>
											<tr>
												<td><?php esc_html_e( $name ); ?></td>
												<td><input type="checkbox" name="goodlocal_express_service[<?php esc_attr_e( $code ); ?>][enabled]" <?php checked( ( ! isset( $this->custom_services[ $code ]['enabled'] ) || ! empty( $this->custom_services[ $code ]['enabled'] ) ), true ); ?> /></td>
												<td><input type="text" name="goodlocal_express_service[<?php esc_attr_e( $code ); ?>][additional_charge]" value="<?php esc_attr_e ( isset( $this->custom_services[ $code ]['additional_charge'] ) ? $this->custom_services[ $code ]['additional_charge'] : '' ); ?>" size="4" /></td>
											</tr>
											<?php
										}
									?>
								</tbody>
							</table>
						</td>
					</tr>
					<?php
					return ob_get_clean();
				}

				/**
				 * validate_services_field function.
				 *
				 * @access public
				 * @param mixed $key
				 * @return void
				 */
				public function validate_services_field( $key ) {
					$services         = array();
					$posted_services  = $_POST['goodlocal_express_service'];

					foreach ( $posted_services as $code => $settings ) {
						$sanitized_additional_charge = sanitize_text_field( $settings['additional_charge'] );
						$filter_additional_charge_is_zero = filter_var( $sanitized_additional_charge, FILTER_VALIDATE_INT ) === 0;
						$filter_additional_charge_more_than_zero = filter_var( $sanitized_additional_charge, FILTER_VALIDATE_INT );
						$additional_charge;

						if ( $filter_additional_charge_is_zero || $filter_additional_charge_more_than_zero ) {
							$additional_charge = $sanitized_additional_charge;
						} else {
							$additional_charge = '';
						}

						if ( $additional_charge < 0 ) {
							$additional_charge = '';
						}

						$services[ $code ] = array(
							'enabled'            => isset( $settings['enabled'] ) ? true : false,
							'additional_charge'  => $additional_charge
						);

					}

					return $services;
				}

				/**
				* Define settings field for this shipping.
				*
				* @return void
				*/
				public function init_form_fields() {

					$this->instance_form_fields = array(
						'title'         => array(
							'title'       => __( 'GoodLocal Facility', 'goodlocal_express_local' ),
							'label'       => __('Title', 'woocommerce'),
							'placeholder' => _x('', 'placeholder', 'woocommerce'),
							'required'    => true,
							'clear'       => false,
							'type'        => 'select',
							'options'     => array(
								'GoodLocal Express' 	=> __('Winnipeg - 1041 Erin St, R3G2X1', 'woocommerce' ),
								// 'GoodLocal Express'	=> __('Next Facility', 'woocommerce' )
								)
						),
						'services' => array(
							'type' => 'services'
						),
					);

				}

				/**
				 * calculate_shipping function.
				 *
				 * @access public
				 * @param mixed $package
				 * @return void
				 */
				public function calculate_shipping( $package = array() ) {
					$customer_postal_code = $package[ 'destination' ][ 'postcode' ];
					$local_postal_codes = array(
						'R3P', 'R3B', 'R2H', 'R2J', 'R2K', 'R2L',
						'R2M', 'R2N', 'R2P', 'R2R', 'R2V', 'R2W',
						'R2X', 'R2Y', 'R3A', 'R3Y', 'R3E', 'R3X',
						'R3W', 'R3T', 'R3S', 'R3R', 'R2G', 'R3N',
						'R3M', 'R3L', 'R3K', 'R3J', 'R3H', 'R3G',
						'R2C', 'R3C'
					);
					$rural_postal_codes = array(
						'R4G', 'R4H', 'R4J', 'R3V', 'R2E', 'R4A'
					);

					if ( $this->custom_services[ "GLE.LD" ]['enabled'] ) {

						if ( empty( $this->custom_services[ "GLE.LD" ]['additional_charge'] ) ) {
							$this->custom_services[ "GLE.LD" ]['additional_charge'] = 0;
						}

						$rate = array(
							'id'				=> 100,
							'label'			=> "GoodLocal Express - Local Delivery",
							'cost'			=> 5.97 + $this->custom_services[ "GLE.LD" ]['additional_charge'],
							'calc_tax'	=> 'per_item'
						);

						// Register the rate
						foreach($local_postal_codes as $local_postal_code) {
							if ( str_contains($customer_postal_code, $local_postal_code) ) {
								$this->add_rate( $rate );
							}
						}
					}

					if ( $this->custom_services[ "GLE.RD" ]['enabled'] ) {

						if ( empty( $this->custom_services[ "GLE.RD" ]['additional_charge'] ) ) {
							$this->custom_services[ "GLE.RD" ]['additional_charge'] = 0;
						}

						$rate = array(
							'id'				=> 200,
							'label'			=> "GoodLocal Express - Rural Delivery",
							'cost'			=> 7.97 + $this->custom_services[ "GLE.RD" ]['additional_charge'],
							'calc_tax'	=> 'per_item'
						);

						// Register the rate
						foreach($rural_postal_codes as $rural_postal_code) {
							if ( str_contains($customer_postal_code, $rural_postal_code) ) {
								$this->add_rate( $rate );
							}
						}
					}
				}

			}
		}
	}

	add_action( 'woocommerce_shipping_init', 'goodlocal_express_shipping_method_init' );

	function goodlocal_express_shipping_method( $methods ) {
		$methods['goodlocal_express_local'] = 'WC_GL_Express_Local_Shipping_Method';

		return $methods;
	}

	add_filter( 'woocommerce_shipping_methods', 'goodlocal_express_shipping_method' );
}
